@@ -30,7 +30,7 @@ M.core_availability.form = {
      * @property plugins
      * @type Object
      */
-    plugins : {},
+    plugins: {},
 
     /**
      * Availability field (textarea).
@@ -38,7 +38,7 @@ M.core_availability.form = {
      * @property field
      * @type Y.Node
      */
-    field : null,
+    field: null,
 
     /**
      * Main div that replaces the availability field.
@@ -46,7 +46,7 @@ M.core_availability.form = {
      * @property mainDiv
      * @type Y.Node
      */
-    mainDiv : null,
+    mainDiv: null,
 
     /**
      * Object that represents the root of the tree.
@@ -54,7 +54,7 @@ M.core_availability.form = {
      * @property rootList
      * @type M.core_availability.List
      */
-    rootList : null,
+    rootList: null,
 
     /**
      * Counter used when creating anything that needs an id.
@@ -62,7 +62,15 @@ M.core_availability.form = {
      * @property idCounter
      * @type Number
      */
-    idCounter : 0,
+    idCounter: 0,
+
+    /**
+     * The 'Restrict by group' button if present.
+     *
+     * @property restrictByGroup
+     * @type Y.Node
+     */
+    restrictByGroup: null,
 
     /**
      * Called to initialise the system when the page loads. This method will
@@ -70,9 +78,9 @@ M.core_availability.form = {
      *
      * @method init
      */
-    init : function(pluginParams) {
+    init: function(pluginParams) {
         // Init all plugins.
-        for(var plugin in pluginParams) {
+        for (var plugin in pluginParams) {
             var params = pluginParams[plugin];
             var pluginClass = M[params[0]].form;
             pluginClass.init.apply(pluginClass, params);
@@ -98,7 +106,7 @@ M.core_availability.form = {
         if (value !== '') {
             try {
                 data = Y.JSON.parse(value);
-            } catch(x) {
+            } catch (x) {
                 // If the JSON data is not valid, treat it as empty.
                 this.field.set('value', '');
             }
@@ -119,6 +127,22 @@ M.core_availability.form = {
         this.field.ancestor('form').on('submit', function() {
             this.mainDiv.all('input,textarea,select').set('disabled', true);
         }, this);
+
+        // If the form has group mode and/or grouping options, there is a
+        // 'add restriction' button there.
+        this.restrictByGroup = Y.one('#restrictbygroup');
+        if (this.restrictByGroup) {
+            this.restrictByGroup.on('click', this.addRestrictByGroup, this);
+            var groupmode = Y.one('#id_groupmode');
+            var groupingid = Y.one('#id_groupingid');
+            if (groupmode) {
+                groupmode.on('change', this.updateRestrictByGroup, this);
+            }
+            if (groupingid) {
+                groupingid.on('change', this.updateRestrictByGroup, this);
+            }
+            this.updateRestrictByGroup();
+        }
     },
 
     /**
@@ -128,7 +152,7 @@ M.core_availability.form = {
      *
      * @method update
      */
-    update : function() {
+    update: function() {
         // Convert tree to value.
         var jsValue = this.rootList.getValue();
 
@@ -141,6 +165,75 @@ M.core_availability.form = {
 
         // Set into hidden form field, JS-encoded.
         this.field.set('value', Y.JSON.stringify(jsValue));
+
+        // Also update the restrict by group button if present.
+        this.updateRestrictByGroup();
+    },
+
+    /**
+     * Updates the status of the 'restrict by group' button (enables or disables
+     * it) based on current availability restrictions and group/grouping settings.
+     */
+    updateRestrictByGroup: function() {
+        if (!this.restrictByGroup) {
+            return;
+        }
+
+        // If the root list is anything other than the default 'and' type, disable.
+        if (this.rootList.getValue().op !== '&') {
+            this.restrictByGroup.set('disabled', true);
+            return;
+        }
+
+        // If there's already a group restriction, disable it.
+        var alreadyGot = this.rootList.hasItemOfType('group') ||
+                this.rootList.hasItemOfType('grouping');
+        if (alreadyGot) {
+            this.restrictByGroup.set('disabled', true);
+            return;
+        }
+
+        // If the groupmode and grouping id aren't set, disable it.
+        var groupmode = Y.one('#id_groupmode');
+        var groupingid = Y.one('#id_groupingid');
+        if ((!groupmode || Number(groupmode.get('value')) === 0) &&
+                (!groupingid || Number(groupingid.get('value')) === 0)) {
+            this.restrictByGroup.set('disabled', true);
+            return;
+        }
+
+        this.restrictByGroup.set('disabled', false);
+    },
+
+    /**
+     * Called when the user clicks on the 'restrict by group' button. This is
+     * a special case that adds a group or grouping restriction.
+     *
+     * By default this restriction is not shown which makes it similar to the
+     *
+     * @param e Button click event
+     */
+    addRestrictByGroup: function(e) {
+        // If you don't prevent default, it submits the form for some reason.
+        e.preventDefault();
+
+        // Add the condition.
+        var groupingid = Y.one('#id_groupingid');
+        var newChild;
+        if (groupingid && Number(groupingid.get('value')) !== 0) {
+            // Add a grouping restriction if one is specified.
+            newChild = new M.core_availability.Item(
+                    {type: 'grouping', id: Number(groupingid.get('value'))}, true);
+        } else {
+            // Otherwise just add a group restriction.
+            newChild = new M.core_availability.Item({type: 'group'}, true);
+        }
+
+        // Refresh HTML.
+        this.rootList.addChild(newChild);
+        this.update();
+        this.rootList.renumber();
+        this.rootList.updateHtml();
     }
 };
 
@@ -158,7 +251,7 @@ M.core_availability.plugin = {
      * @property allowAdd
      * @type Boolean
      */
-    allowAdd : false,
+    allowAdd: false,
 
     /**
      * Called (from PHP) to initialise the plugin. Should usually not be
@@ -167,7 +260,7 @@ M.core_availability.plugin = {
      * @method init
      * @param {String} component Component name e.g. 'availability_date'
      */
-    init : function(component, allowAdd, params) {
+    init: function(component, allowAdd, params) {
         var name = component.replace(/^availability_/, '');
         this.allowAdd = allowAdd;
         M.core_availability.form.plugins[name] = this;
@@ -183,7 +276,8 @@ M.core_availability.plugin = {
      * @method initInner
      * @protected
      */
-    initInner : function() {
+    initInner: function() {
+        // Can be overriden.
     },
 
     /**
@@ -194,7 +288,7 @@ M.core_availability.plugin = {
      * @method getNode
      * @return {Y.Node} YUI node
      */
-    getNode : function() {
+    getNode: function() {
         throw 'getNode not implemented';
     },
 
@@ -208,7 +302,7 @@ M.core_availability.plugin = {
      * @param {Object} value Value object (to be written to)
      * @param {Y.Node} node YUI node (same one returned from getNode)
      */
-    fillValue : function() {
+    fillValue: function() {
         throw 'fillValue not implemented';
     },
 
@@ -225,7 +319,8 @@ M.core_availability.plugin = {
      * @param {Array} errors Array of errors (push new errors here)
      * @param {Y.Node} node YUI node (same one returned from getNode)
      */
-    fillErrors : function() {
+    fillErrors: function() {
+        // Can be overriden.
     },
 
     /**
@@ -234,7 +329,7 @@ M.core_availability.plugin = {
      * The default implementation uses a simple algorithm to identify the
      * first focusable input/select and then focuses it.
      */
-    focusAfterAdd : function(node) {
+    focusAfterAdd: function(node) {
         var target = node.one('input:not([disabled]),select:not([disabled])');
         target.focus();
     }
@@ -258,27 +353,30 @@ M.core_availability.List = function(json, root, parentRoot) {
     if (root !== undefined) {
         this.root = root;
     }
-    var strings = M.str.availability;
     // Create DIV structure (without kids).
     this.node = Y.Node.create('<div class="availability-list"><h3 class="accesshide"></h3>' +
             '<div class="availability-inner">' +
-            '<div class="availability-header">' + strings.listheader_sign_before +
-            ' <label><span class="accesshide">' + strings.label_sign +
-            ' </span><select class="availability-neg" title="' + strings.label_sign + '">' +
-            '<option value="">' + strings.listheader_sign_pos + '</option>' +
-            '<option value="!">' + strings.listheader_sign_neg + '</option></select></label> ' +
-            '<span class="availability-single">' + strings.listheader_single + '</span>' +
-            '<span class="availability-multi">' + strings.listheader_multi_before +
-            ' <label><span class="accesshide">' + strings.label_multi + ' </span>' +
-            '<select class="availability-op" title="' + strings.label_multi + '"><option value="&">' +
-            strings.listheader_multi_and + '</option>' +
-            '<option value="|">' + strings.listheader_multi_or + '</option></select></label> ' +
-            strings.listheader_multi_after + '</span></div>' +
+            '<div class="availability-header mb-1"><span>' +
+            M.util.get_string('listheader_sign_before', 'availability') + '</span>' +
+            ' <label><span class="accesshide">' + M.util.get_string('label_sign', 'availability') +
+            ' </span><select class="availability-neg custom-select mx-1"' +
+            ' title="' + M.util.get_string('label_sign', 'availability') + '">' +
+            '<option value="">' + M.util.get_string('listheader_sign_pos', 'availability') + '</option>' +
+            '<option value="!">' + M.util.get_string('listheader_sign_neg', 'availability') + '</option></select></label> ' +
+            '<span class="availability-single">' + M.util.get_string('listheader_single', 'availability') + '</span>' +
+            '<span class="availability-multi">' + M.util.get_string('listheader_multi_before', 'availability') +
+            ' <label><span class="accesshide">' + M.util.get_string('label_multi', 'availability') + ' </span>' +
+            '<select class="availability-op custom-select mx-1"' +
+            ' title="' + M.util.get_string('label_multi', 'availability') + '"><option value="&">' +
+            M.util.get_string('listheader_multi_and', 'availability') + '</option>' +
+            '<option value="|">' + M.util.get_string('listheader_multi_or', 'availability') + '</option></select></label> ' +
+            M.util.get_string('listheader_multi_after', 'availability') + '</span></div>' +
             '<div class="availability-children"></div>' +
-            '<div class="availability-none">' + M.str.moodle.none + '</div>' +
-            '<div class="availability-button"></div></div></div>');
+            '<div class="availability-none"><span class="p-x-1">' + M.util.get_string('none', 'moodle') + '</span></div>' +
+            '<div class="clearfix mt-1"></div>' +
+            '<div class="availability-button"></div></div><div class="clearfix"></div></div>');
     if (!root) {
-        this.node.addClass('availability-childlist');
+        this.node.addClass('availability-childlist d-sm-flex align-items-center');
     }
     this.inner = this.node.one('> .availability-inner');
 
@@ -310,14 +408,16 @@ M.core_availability.List = function(json, root, parentRoot) {
         noneNode.appendChild(deleteIcon.span);
 
         // Also if it's not the root, none is actually invalid, so add a label.
-        noneNode.appendChild(Y.Node.create('<span class="label label-warning">' +
-                M.str.availability.invalid + '</span>'));
+        noneNode.appendChild(Y.Node.create('<span class="mt-1 badge badge-warning">' +
+                M.util.get_string('invalid', 'availability') + '</span>'));
     }
 
     // Create the button and add it.
-    var button = Y.Node.create('<button type="button" class="btn btn-default">' +
-            M.str.availability.addrestriction + '</button>');
-    button.on("click", function() { this.clickAdd(); }, this);
+    var button = Y.Node.create('<button type="button" class="btn btn-secondary mt-1">' +
+            M.util.get_string('addrestriction', 'availability') + '</button>');
+    button.on("click", function() {
+        this.clickAdd();
+    }, this);
     this.node.one('div.availability-button').appendChild(button);
 
     if (json) {
@@ -438,7 +538,7 @@ M.core_availability.List.prototype.isIndividualShowIcons = function() {
  */
 M.core_availability.List.prototype.renumber = function(parentNumber) {
     // Update heading for list.
-    var headingParams = { count: this.children.length };
+    var headingParams = {count: this.children.length};
     var prefix;
     if (parentNumber === undefined) {
         headingParams.number = '';
@@ -507,9 +607,9 @@ M.core_availability.List.prototype.updateHtml = function() {
     // Update connector text.
     var connectorText;
     if (this.inner.one('.availability-op').get('value') === '&') {
-        connectorText = M.str.availability.and;
+        connectorText = M.util.get_string('and', 'availability');
     } else {
-        connectorText = M.str.availability.or;
+        connectorText = M.util.get_string('or', 'availability');
     }
     this.inner.all('> .availability-children > .availability-connector span.label').each(function(span) {
         span.set('innerHTML', connectorText);
@@ -569,14 +669,14 @@ M.core_availability.List.prototype.deleteDescendant = function(descendant) {
  */
 M.core_availability.List.prototype.clickAdd = function() {
     var content = Y.Node.create('<div>' +
-            '<ul class="list-unstyled"></ul>' +
+            '<ul class="list-unstyled container-fluid"></ul>' +
             '<div class="availability-buttons mdl-align">' +
-            '<button type="button" class="btn btn-default">' + M.str.moodle.cancel +
+            '<button type="button" class="btn btn-secondary">' + M.util.get_string('cancel', 'moodle') +
             '</button></div></div>');
     var cancel = content.one('button');
 
     // Make a list of all the dialog options.
-    var dialogRef = { dialog: null };
+    var dialogRef = {dialog: null};
     var ul = content.one('ul');
     var li, id, button, label;
     for (var type in M.core_availability.form.plugins) {
@@ -585,38 +685,37 @@ M.core_availability.List.prototype.clickAdd = function() {
             continue;
         }
         // Add entry for plugin.
-        li = Y.Node.create('<li class="clearfix"></li>');
+        li = Y.Node.create('<li class="clearfix row"></li>');
         id = 'availability_addrestriction_' + type;
-        var pluginStrings = M.str['availability_' + type];
-        button = Y.Node.create('<button type="button" class="btn btn-default"' +
-                'id="' + id + '">' + pluginStrings.title + '</button>');
+        button = Y.Node.create('<div class="col-6"><button type="button" class="btn btn-secondary w-100"' +
+                'id="' + id + '">' + M.util.get_string('title', 'availability_' + type) + '</button></div>');
         button.on('click', this.getAddHandler(type, dialogRef), this);
         li.appendChild(button);
-        label = Y.Node.create('<label for="' + id + '">' +
-                pluginStrings.description + '</label>');
+        label = Y.Node.create('<div class="col-6"><label for="' + id + '">' +
+                M.util.get_string('description', 'availability_' + type) + '</label></div>');
         li.appendChild(label);
         ul.appendChild(li);
     }
     // Extra entry for lists.
-    li = Y.Node.create('<li class="clearfix"></li>');
+    li = Y.Node.create('<li class="clearfix row"></li>');
     id = 'availability_addrestriction_list_';
-    button = Y.Node.create('<button type="button" class="btn btn-default"' +
-            'id="' + id + '">' + M.str.availability.condition_group + '</button>');
+    button = Y.Node.create('<div class="col-6"><button type="button" class="btn btn-secondary w-100"' +
+            'id="' + id + '">' + M.util.get_string('condition_group', 'availability') + '</button></div>');
     button.on('click', this.getAddHandler(null, dialogRef), this);
     li.appendChild(button);
-    label = Y.Node.create('<label for="' + id + '">' +
-            M.str.availability.condition_group_info + '</label>');
+    label = Y.Node.create('<div class="col-6"><label for="' + id + '">' +
+            M.util.get_string('condition_group_info', 'availability') + '</label></div>');
     li.appendChild(label);
     ul.appendChild(li);
 
     var config = {
-        headerContent : M.str.availability.addrestriction,
-        bodyContent : content,
-        additionalBaseClass : 'availability-dialogue',
-        draggable : true,
-        modal : true,
-        closeButton : false,
-        width : '450px'
+        headerContent: M.util.get_string('addrestriction', 'availability'),
+        bodyContent: content,
+        additionalBaseClass: 'availability-dialogue',
+        draggable: true,
+        modal: true,
+        closeButton: false,
+        width: '450px'
     };
     dialogRef.dialog = new M.core.dialogue(config);
     dialogRef.dialog.show();
@@ -638,12 +737,13 @@ M.core_availability.List.prototype.clickAdd = function() {
  */
 M.core_availability.List.prototype.getAddHandler = function(type, dialogRef) {
     return function() {
+        var newItem;
         if (type) {
             // Create an Item object to represent the child.
-            newItem = new M.core_availability.Item({ type: type, creating: true }, this.root);
+            newItem = new M.core_availability.Item({type: type, creating: true}, this.root);
         } else {
             // Create a new List object to represent the child.
-            newItem = new M.core_availability.List({ c: [], showc: true }, false, this.root);
+            newItem = new M.core_availability.List({c: [], showc: true}, false, this.root);
         }
         // Add to list.
         this.addChild(newItem);
@@ -709,6 +809,31 @@ M.core_availability.List.prototype.fillErrors = function(errors) {
     }
 };
 
+/**
+ * Checks whether the list contains any items of the given type name.
+ *
+ * @method hasItemOfType
+ * @param {String} pluginType Required plugin type (name)
+ * @return {Boolean} True if there is one
+ */
+M.core_availability.List.prototype.hasItemOfType = function(pluginType) {
+    // Check each item.
+    for (var i = 0; i < this.children.length; i++) {
+        var child = this.children[i];
+        if (child instanceof M.core_availability.List) {
+            // Recursive call.
+            if (child.hasItemOfType(pluginType)) {
+                return true;
+            }
+        } else {
+            if (child.pluginType === pluginType) {
+                return true;
+            }
+        }
+    }
+    return false;
+};
+
 /**
  * Eye icon for this list (null if none).
  *
@@ -764,7 +889,7 @@ M.core_availability.Item = function(json, root) {
         // Handle undefined plugins.
         this.plugin = null;
         this.pluginNode = Y.Node.create('<div class="availability-warning">' +
-                M.str.availability.missingplugin + '</div>');
+                M.util.get_string('missingplugin', 'availability') + '</div>');
     } else {
         // Plugin is known.
         this.plugin = M.core_availability.form.plugins[json.type];
@@ -774,13 +899,13 @@ M.core_availability.Item = function(json, root) {
         this.pluginNode.addClass('availability_' + json.type);
     }
 
-    this.node = Y.Node.create('<div class="availability-item"><h3 class="accesshide"></h3></div>');
+    this.node = Y.Node.create('<div class="availability-item d-sm-flex align-items-center"><h3 class="accesshide"></h3></div>');
 
     // Add eye icon if required. This icon is added for root items, but may be
     // hidden depending on the selected list operator.
     if (root) {
         var shown = true;
-        if(json.showc !== undefined) {
+        if (json.showc !== undefined) {
             shown = json.showc;
         }
         this.eyeIcon = new M.core_availability.EyeIcon(true, shown);
@@ -797,7 +922,7 @@ M.core_availability.Item = function(json, root) {
 
     // Add the invalid marker (empty).
     this.node.appendChild(document.createTextNode(' '));
-    this.node.appendChild(Y.Node.create('<span class="label label-warning"/>'));
+    this.node.appendChild(Y.Node.create('<span class="badge badge-warning"/>'));
 };
 
 /**
@@ -808,7 +933,7 @@ M.core_availability.Item = function(json, root) {
  * @return {Object} JavaScript object containing value of this item
  */
 M.core_availability.Item.prototype.getValue = function() {
-    value = { 'type' : this.pluginType };
+    var value = {'type': this.pluginType};
     if (this.plugin) {
         this.plugin.fillValue(value, this.pluginNode);
     }
@@ -833,9 +958,9 @@ M.core_availability.Item.prototype.fillErrors = function(errors) {
         errors.push('core_availability:item_unknowntype');
     }
     // If any errors were added, add the marker to this item.
-    var errorLabel = this.node.one('> .label-warning');
+    var errorLabel = this.node.one('> .badge-warning');
     if (errors.length !== before && !errorLabel.get('firstChild')) {
-        errorLabel.appendChild(document.createTextNode(M.str.availability.invalid));
+        errorLabel.appendChild(document.createTextNode(M.util.get_string('invalid', 'availability')));
     } else if (errors.length === before && errorLabel.get('firstChild')) {
         errorLabel.get('firstChild').remove();
     }
@@ -849,9 +974,9 @@ M.core_availability.Item.prototype.fillErrors = function(errors) {
  */
 M.core_availability.Item.prototype.renumber = function(number) {
     // Update heading for item.
-    var headingParams = { number: number };
+    var headingParams = {number: number};
     if (this.plugin) {
-        headingParams.type = M.str['availability_' + this.pluginType].title;
+        headingParams.type = M.util.get_string('title', 'availability_' + this.pluginType);
     } else {
         headingParams.type = '[' + this.pluginType + ']';
     }
@@ -925,26 +1050,27 @@ M.core_availability.Item.prototype.pluginNode = null;
  */
 M.core_availability.EyeIcon = function(individual, shown) {
     this.individual = individual;
-    this.span = Y.Node.create('<a class="availability-eye" href="#" role="button">');
-    var iconBase = M.cfg.wwwroot + '/theme/image.php/' + M.cfg.theme + '/core/' + M.cfg.themerev;
+    this.span = Y.Node.create('<a class="availability-eye col-form-label" href="#" role="button">');
     var icon = Y.Node.create('<img />');
     this.span.appendChild(icon);
 
     // Set up button text and icon.
-    var suffix = individual ? '_individual' : '_all';
-    var setHidden = function() {
-        icon.set('src', iconBase + '/i/show');
-        icon.set('alt', M.str.availability['hidden' + suffix]);
-        this.span.set('title', M.str.availability['hidden' + suffix] + ' \u2022 ' +
-                M.str.availability.show_verb);
-    };
-    var setShown = function() {
-        icon.set('src', iconBase + '/i/hide');
-        icon.set('alt', M.str.availability['shown' + suffix]);
-        this.span.set('title', M.str.availability['shown' + suffix] + ' \u2022 ' +
-                M.str.availability.hide_verb);
-    };
-    if(shown) {
+    var suffix = individual ? '_individual' : '_all',
+        setHidden = function() {
+            var hiddenStr = M.util.get_string('hidden' + suffix, 'availability');
+            icon.set('src', M.util.image_url('i/show', 'core'));
+            icon.set('alt', hiddenStr);
+            this.span.set('title', hiddenStr + ' \u2022 ' +
+                    M.util.get_string('show_verb', 'availability'));
+        },
+        setShown = function() {
+            var shownStr = M.util.get_string('shown' + suffix, 'availability');
+            icon.set('src', M.util.image_url('i/hide', 'core'));
+            icon.set('alt', shownStr);
+            this.span.set('title', shownStr + ' \u2022 ' +
+                    M.util.get_string('hide_verb', 'availability'));
+        };
+    if (shown) {
         setShown.call(this);
     } else {
         setHidden.call(this);
@@ -962,7 +1088,9 @@ M.core_availability.EyeIcon = function(individual, shown) {
     };
     this.span.on('click', click, this);
     this.span.on('key', click, 'up:32', this);
-    this.span.on('key', function(e) { e.preventDefault(); }, 'down:32', this);
+    this.span.on('key', function(e) {
+        e.preventDefault();
+    }, 'down:32', this);
 };
 
 /**
@@ -988,8 +1116,8 @@ M.core_availability.EyeIcon.prototype.span = null;
  * @return {Boolean} True if this icon is set to 'hidden'
  */
 M.core_availability.EyeIcon.prototype.isHidden = function() {
-    var suffix = this.individual ? '_individual' : '_all';
-    var compare = M.str.availability['hidden' + suffix];
+    var suffix = this.individual ? '_individual' : '_all',
+        compare = M.util.get_string('hidden' + suffix, 'availability');
     return this.span.one('img').get('alt') === compare;
 };
 
@@ -1002,11 +1130,10 @@ M.core_availability.EyeIcon.prototype.isHidden = function() {
  * @param {M.core_availability.Item|M.core_availability.List} toDelete Thing to delete
  */
 M.core_availability.DeleteIcon = function(toDelete) {
-    this.span = Y.Node.create('<a class="availability-delete" href="#" title="' +
-            M.str.moodle['delete'] + '" role="button">');
-    var img = Y.Node.create('<img src="' +
-            M.cfg.wwwroot + '/theme/image.php/' + M.cfg.theme + '/core/' + M.cfg.themerev +
-            '/t/delete" alt="' + M.str.moodle['delete'] + '" />');
+    this.span = Y.Node.create('<a class="d-inline-block col-form-label availability-delete p-x-1" href="#" title="' +
+            M.util.get_string('delete', 'moodle') + '" role="button">');
+    var img = Y.Node.create('<img src="' + M.util.image_url('t/delete', 'core') +
+            '" alt="' + M.util.get_string('delete', 'moodle') + '" />');
     this.span.appendChild(img);
     var click = function(e) {
         e.preventDefault();
@@ -1015,7 +1142,9 @@ M.core_availability.DeleteIcon = function(toDelete) {
     };
     this.span.on('click', click, this);
     this.span.on('key', click, 'up:32', this);
-    this.span.on('key', function(e) { e.preventDefault(); }, 'down:32', this);
+    this.span.on('key', function(e) {
+        e.preventDefault();
+    }, 'down:32', this);
 };
 
 /**
